@@ -6,6 +6,7 @@ import { FetchResponse } from '@grafana/runtime';
 import config from 'app/core/config';
 import { backendSrv } from 'app/core/services/backend_srv'; // will use the version in __mocks__
 
+import { BROWSER_MODE_DISABLED_MESSAGE } from '../constants';
 import InfluxDatasource from '../datasource';
 
 //@ts-ignore
@@ -26,6 +27,7 @@ describe('InfluxDataSource', () => {
   beforeEach(() => {
     jest.clearAllMocks();
     ctx.instanceSettings.url = '/api/datasources/proxy/1';
+    ctx.instanceSettings.access = 'proxy';
     ctx.ds = new InfluxDatasource(ctx.instanceSettings, templateSrv);
   });
 
@@ -114,6 +116,8 @@ describe('InfluxDataSource', () => {
         } as FetchResponse);
       });
 
+      ctx.ds.retentionPolicies = [''];
+
       try {
         await lastValueFrom(ctx.ds.query(queryOptions));
       } catch (err) {
@@ -124,6 +128,25 @@ describe('InfluxDataSource', () => {
     });
   });
 
+  describe('When getting a request after issuing a query using outdated Browser Mode', () => {
+    beforeEach(() => {
+      jest.clearAllMocks();
+      ctx.instanceSettings.url = '/api/datasources/proxy/1';
+      ctx.instanceSettings.access = 'direct';
+      ctx.ds = new InfluxDatasource(ctx.instanceSettings, templateSrv);
+    });
+
+    it('throws an error', async () => {
+      try {
+        await lastValueFrom(ctx.ds.query({}));
+      } catch (err) {
+        if (err instanceof Error) {
+          expect(err.message).toBe(BROWSER_MODE_DISABLED_MESSAGE);
+        }
+      }
+    });
+  });
+
   describe('InfluxDataSource in POST query mode', () => {
     const ctx: any = {
       instanceSettings: { url: 'url', name: 'influxDb', jsonData: { httpMode: 'POST' } },
@@ -179,6 +202,17 @@ describe('InfluxDataSource', () => {
     });
   });
 
+  // Some functions are required by the parent datasource class to provide functionality
+  // such as ad-hoc filters, which requires the definition of the getTagKeys, and getTagValues
+  describe('Datasource contract', () => {
+    it('has function called getTagKeys', () => {
+      expect(Object.getOwnPropertyNames(Object.getPrototypeOf(ctx.ds))).toContain('getTagKeys');
+    });
+    it('has function called getTagValues', () => {
+      expect(Object.getOwnPropertyNames(Object.getPrototypeOf(ctx.ds))).toContain('getTagValues');
+    });
+  });
+
   describe('Variables should be interpolated correctly', () => {
     const instanceSettings: any = {};
     const text = 'interpolationText';
@@ -189,7 +223,18 @@ describe('InfluxDataSource', () => {
       $interpolationVar: text,
       $interpolationVar2: text2,
     };
+    const adhocFilters = [
+      {
+        key: 'adhoc',
+        operator: '=',
+        value: 'val',
+        condition: '',
+      },
+    ];
     const templateSrv: any = {
+      getAdhocFilters: jest.fn((name: string) => {
+        return adhocFilters;
+      }),
       replace: jest.fn((target?: string, scopedVars?: ScopedVars, format?: string | Function): string => {
         if (!format) {
           return variableMap[target!] || '';
@@ -231,6 +276,7 @@ describe('InfluxDataSource', () => {
           },
         ],
       ],
+      adhocFilters,
     };
 
     function influxChecks(query: any) {
@@ -244,6 +290,7 @@ describe('InfluxDataSource', () => {
       expect(query.tags![0].value).toBe(textWithFormatRegex);
       expect(query.groupBy![0].params![0]).toBe(textWithFormatRegex);
       expect(query.select![0][0].params![0]).toBe(textWithFormatRegex);
+      expect(query.adhocFilters[0].key).toBe(adhocFilters[0].key);
     }
 
     describe('when interpolating query variables for dashboard->explore', () => {
@@ -298,6 +345,19 @@ describe('InfluxDataSource', () => {
         });
         influxChecks(query);
       });
+
+      it('should apply all scopedVars to tags', () => {
+        ds.isFlux = false;
+        ds.access = 'proxy';
+        config.featureToggles.influxdbBackendMigration = true;
+        const query = ds.applyTemplateVariables(influxQuery, {
+          interpolationVar: { text: text, value: text },
+          interpolationVar2: { text: 'interpolationText2', value: 'interpolationText2' },
+        });
+        const value = query.tags[0].value;
+        const scopedVars = 'interpolationText|interpolationText2';
+        expect(value).toBe(scopedVars);
+      });
     });
   });
 });
