@@ -1,9 +1,8 @@
 import { descending, deviation } from 'd3';
-import { groupBy, partition } from 'lodash';
+import { flatten, forOwn, groupBy, partition } from 'lodash';
 
 import {
   ArrayDataFrame,
-  ArrayVector,
   CoreApp,
   DataFrame,
   DataFrameType,
@@ -16,13 +15,13 @@ import {
   formatLabels,
   getDisplayProcessor,
   Labels,
-  MutableField,
   PreferredVisualisationType,
   ScopedVars,
   TIME_SERIES_TIME_FIELD_NAME,
   TIME_SERIES_VALUE_FIELD_NAME,
 } from '@grafana/data';
-import { FetchResponse, getDataSourceSrv, getTemplateSrv } from '@grafana/runtime';
+import { calculateFieldDisplayName } from '@grafana/data/src/field/fieldState';
+import { config, FetchResponse, getDataSourceSrv, getTemplateSrv } from '@grafana/runtime';
 
 import { renderLegendFormat } from './legend';
 import {
@@ -65,12 +64,29 @@ const isHeatmapResult = (dataFrame: DataFrame, options: DataQueryRequest<PromQue
   return target?.format === 'heatmap';
 };
 
-// V2 result trasnformer used to transform query results from queries that were run trough prometheus backend
+// V2 result transformer used to transform query results from queries that were run through prometheus backend
 export function transformV2(
   response: DataQueryResponse,
   request: DataQueryRequest<PromQuery>,
   options: { exemplarTraceIdDestinations?: ExemplarTraceIdDestination[] }
 ) {
+  // migration for dataplane field name issue
+  if (config.featureToggles.prometheusDataplane) {
+    // update displayNameFromDS in the field config
+    response.data.forEach((f: DataFrame) => {
+      const target = request.targets.find((t) => t.refId === f.refId);
+      // check that the legend is selected as auto
+      if (target && target.legendFormat === '__auto') {
+        f.fields.forEach((field) => {
+          if (field.labels?.__name__ && field.labels?.__name__ === field.name) {
+            const fieldCopy = { ...field, name: TIME_SERIES_VALUE_FIELD_NAME };
+            field.config.displayNameFromDS = calculateFieldDisplayName(fieldCopy, f, response.data);
+          }
+        });
+      }
+    });
+  }
+
   const [tableFrames, framesWithoutTable] = partition<DataFrame>(response.data, (df) => isTableResult(df, request));
   const processedTableFrames = transformDFToTable(tableFrames);
 
@@ -102,28 +118,80 @@ export function transformV2(
     (df) => isHeatmapResult(df, request)
   );
 
-  const processedHeatmapFrames = mergeHeatmapFrames(
-    transformToHistogramOverTime(heatmapResults.sort(sortSeriesByLabel))
-  );
+  // this works around the fact that we only get back frame.name with le buckets when legendFormat == {{le}}...which is not the default
+  heatmapResults.forEach((df) => {
+    if (df.name == null) {
+      let f = df.fields.find((f) => f.name === 'Value');
+
+      if (f) {
+        let le = f.labels?.le;
+
+        if (le) {
+          // this is used for sorting the frames by numeric ascending le labels for de-accum
+          df.name = le;
+          // this is used for renaming the Value fields to le label
+          f.config.displayNameFromDS = le;
+        }
+      }
+    }
+  });
+
+  // Group heatmaps by query
+  const heatmapResultsGroupedByQuery = groupBy<DataFrame>(heatmapResults, (h) => h.refId);
+
+  // Initialize empty array to push grouped histogram frames to
+  let processedHeatmapResultsGroupedByQuery: DataFrame[][] = [];
+
+  // Iterate through every query in this heatmap
+  for (const query in heatmapResultsGroupedByQuery) {
+    // Get reference to dataFrames for heatmap
+    const heatmapResultsGroup = heatmapResultsGroupedByQuery[query];
+
+    // Create a new grouping by iterating through the data frames...
+    const heatmapResultsGroupedByValues = groupBy<DataFrame>(heatmapResultsGroup, (dataFrame) => {
+      // Each data frame has `Time` and `Value` properties, we want to get the values
+      const values = dataFrame.fields.find((field) => field.name === TIME_SERIES_VALUE_FIELD_NAME);
+      // Specific functionality for special "le" quantile heatmap value, we know if this value exists, that we do not want to calculate the heatmap density across data frames from the same quartile
+      if (values?.labels && HISTOGRAM_QUANTILE_LABEL_NAME in values.labels) {
+        const { le, ...notLE } = values?.labels;
+        return Object.values(notLE).join();
+      }
+
+      // Return a string made from the concatenation of this frame's values to represent a grouping in the query
+      return Object.values(values?.labels ?? []).join();
+    });
+
+    // Then iterate through the resultant object
+    forOwn(heatmapResultsGroupedByValues, (dataFrames, key) => {
+      // Sort frames within each grouping
+      const sortedHeatmap = dataFrames.sort(sortSeriesByLabel);
+      // And push the sorted grouping with the rest
+      processedHeatmapResultsGroupedByQuery.push(mergeHeatmapFrames(transformToHistogramOverTime(sortedHeatmap)));
+    });
+  }
 
   // Everything else is processed as time_series result and graph preferredVisualisationType
   const otherFrames = framesWithoutTableHeatmapsAndExemplars.map((dataFrame) => {
-    const df = {
+    const df: DataFrame = {
       ...dataFrame,
       meta: {
         ...dataFrame.meta,
         preferredVisualisationType: 'graph',
       },
-    } as DataFrame;
+    };
     return df;
   });
 
+  const flattenedProcessedHeatmapFrames = flatten(processedHeatmapResultsGroupedByQuery);
+
   return {
     ...response,
-    data: [...otherFrames, ...processedTableFrames, ...processedHeatmapFrames, ...processedExemplarFrames],
+    data: [...otherFrames, ...processedTableFrames, ...flattenedProcessedHeatmapFrames, ...processedExemplarFrames],
   };
 }
 
+const HISTOGRAM_QUANTILE_LABEL_NAME = 'le';
+
 export function transformDFToTable(dfs: DataFrame[]): DataFrame[] {
   // If no dataFrames or if 1 dataFrames with no values, return original dataFrame
   if (dfs.length === 0 || (dfs.length === 1 && dfs[0].length === 0)) {
@@ -139,24 +207,24 @@ export function transformDFToTable(dfs: DataFrame[]): DataFrame[] {
     const valueText = getValueText(refIds.length, refId);
     const valueField = getValueField({ data: [], valueName: valueText });
     const timeField = getTimeField([]);
-    const labelFields: MutableField[] = [];
+    const labelFields: Field[] = [];
 
     // Fill labelsFields with labels from dataFrames
     dataFramesByRefId[refId].forEach((df) => {
       const frameValueField = df.fields[1];
-      const promLabels = frameValueField.labels ?? {};
+      const promLabels = frameValueField?.labels ?? {};
 
       Object.keys(promLabels)
         .sort()
         .forEach((label) => {
           // If we don't have label in labelFields, add it
           if (!labelFields.some((l) => l.name === label)) {
-            const numberField = label === 'le';
+            const numberField = label === HISTOGRAM_QUANTILE_LABEL_NAME;
             labelFields.push({
               name: label,
               config: { filterable: true },
               type: numberField ? FieldType.number : FieldType.string,
-              values: new ArrayVector(),
+              values: [],
             });
           }
         });
@@ -164,11 +232,13 @@ export function transformDFToTable(dfs: DataFrame[]): DataFrame[] {
 
     // Fill valueField, timeField and labelFields with values
     dataFramesByRefId[refId].forEach((df) => {
-      df.fields[0].values.toArray().forEach((value) => timeField.values.add(value));
-      df.fields[1].values.toArray().forEach((value) => {
-        valueField.values.add(parseSampleValue(value));
+      const timeFields = df.fields[0]?.values ?? [];
+      const dataFields = df.fields[1]?.values ?? [];
+      timeFields.forEach((value) => timeField.values.push(value));
+      dataFields.forEach((value) => {
+        valueField.values.push(parseSampleValue(value));
         const labelsForField = df.fields[1].labels ?? {};
-        labelFields.forEach((field) => field.values.add(getLabelValue(labelsForField, field.name)));
+        labelFields.forEach((field) => field.values.push(getLabelValue(labelsForField, field.name)));
       });
     });
 
@@ -176,7 +246,8 @@ export function transformDFToTable(dfs: DataFrame[]): DataFrame[] {
     return {
       refId,
       fields,
-      meta: { ...dfs[0].meta, preferredVisualisationType: 'table' as PreferredVisualisationType },
+      // Prometheus specific UI for instant queries
+      meta: { ...dfs[0].meta, preferredVisualisationType: 'rawPrometheus' as PreferredVisualisationType },
       length: timeField.values.length,
     };
   });
@@ -211,7 +282,7 @@ export function transform(
     valueWithRefId: transformOptions.target.valueWithRefId,
     meta: {
       // Fix for showing of Prometheus results in Explore table
-      preferredVisualisationType: transformOptions.query.instant ? 'table' : 'graph',
+      preferredVisualisationType: transformOptions.query.instant ? 'rawPrometheus' : 'graph',
     },
   };
   const prometheusResult = response.data.data;
@@ -257,14 +328,13 @@ export function transform(
 
   // Return early if result type is scalar
   if (prometheusResult.resultType === 'scalar') {
-    return [
-      {
-        meta: options.meta,
-        refId: options.refId,
-        length: 1,
-        fields: [getTimeField([prometheusResult.result]), getValueField({ data: [prometheusResult.result] })],
-      },
-    ];
+    const df: DataFrame = {
+      meta: options.meta,
+      refId: options.refId,
+      length: 1,
+      fields: [getTimeField([prometheusResult.result]), getValueField({ data: [prometheusResult.result] })],
+    };
+    return [df];
   }
 
   // Return early again if the format is table, this needs special transformation.
@@ -302,7 +372,7 @@ function getDataLinks(options: ExemplarTraceIdDestination): DataLink[] {
         title: options.urlDisplayLabel || `Query with ${dsSettings?.name}`,
         url: '',
         internal: {
-          query: { query: '${__value.raw}', queryType: 'traceId' },
+          query: { query: '${__value.raw}', queryType: 'traceql' },
           datasourceUid: options.datasourceUid,
           datasourceName: dsSettings?.name ?? 'Data source not found',
         },
@@ -441,27 +511,28 @@ function transformMetricDataToTable(md: MatrixOrVectorResult[], options: Transfo
     .map((label) => {
       // Labels have string field type, otherwise table tries to figure out the type which can result in unexpected results
       // Only "le" label has a number field type
-      const numberField = label === 'le';
-      return {
+      const numberField = label === HISTOGRAM_QUANTILE_LABEL_NAME;
+      const field: Field = {
         name: label,
         config: { filterable: true },
         type: numberField ? FieldType.number : FieldType.string,
-        values: new ArrayVector(),
+        values: [],
       };
+      return field;
     });
   const valueField = getValueField({ data: [], valueName: valueText });
 
   md.forEach((d) => {
     if (isMatrixData(d)) {
       d.values.forEach((val) => {
-        timeField.values.add(val[0] * 1000);
-        metricFields.forEach((metricField) => metricField.values.add(getLabelValue(d.metric, metricField.name)));
-        valueField.values.add(parseSampleValue(val[1]));
+        timeField.values.push(val[0] * 1000);
+        metricFields.forEach((metricField) => metricField.values.push(getLabelValue(d.metric, metricField.name)));
+        valueField.values.push(parseSampleValue(val[1]));
       });
     } else {
-      timeField.values.add(d.value[0] * 1000);
-      metricFields.forEach((metricField) => metricField.values.add(getLabelValue(d.metric, metricField.name)));
-      valueField.values.add(parseSampleValue(d.value[1]));
+      timeField.values.push(d.value[0] * 1000);
+      metricFields.forEach((metricField) => metricField.values.push(getLabelValue(d.metric, metricField.name)));
+      valueField.values.push(parseSampleValue(d.value[1]));
     }
   });
 
@@ -475,7 +546,7 @@ function transformMetricDataToTable(md: MatrixOrVectorResult[], options: Transfo
 
 function getLabelValue(metric: PromMetric, label: string): string | number {
   if (metric.hasOwnProperty(label)) {
-    if (label === 'le') {
+    if (label === HISTOGRAM_QUANTILE_LABEL_NAME) {
       return parseSampleValue(metric[label]);
     }
     return metric[label];
@@ -483,12 +554,12 @@ function getLabelValue(metric: PromMetric, label: string): string | number {
   return '';
 }
 
-function getTimeField(data: PromValue[], isMs = false): MutableField {
+function getTimeField(data: PromValue[], isMs = false): Field<number> {
   return {
     name: TIME_SERIES_TIME_FIELD_NAME,
     type: FieldType.time,
     config: {},
-    values: new ArrayVector<number>(data.map((val) => (isMs ? val[0] : val[0] * 1000))),
+    values: data.map((val) => (isMs ? val[0] : val[0] * 1000)),
   };
 }
 
@@ -506,7 +577,7 @@ function getValueField({
   parseValue = true,
   labels,
   displayNameFromDS,
-}: ValueFieldOptions): MutableField {
+}: ValueFieldOptions): Field {
   return {
     name: valueName,
     type: FieldType.number,
@@ -515,7 +586,7 @@ function getValueField({
       displayNameFromDS,
     },
     labels,
-    values: new ArrayVector<number | null>(data.map((val) => (parseValue ? parseSampleValue(val[1]) : val[1]))),
+    values: data.map((val) => (parseValue ? parseSampleValue(val[1]) : val[1])),
   };
 }
 
@@ -587,21 +658,21 @@ function transformToHistogramOverTime(seriesList: DataFrame[]) {
     }
 
     for (let j = 0; j < topSeries.values.length; j++) {
-      const bottomPoint = bottomSeries.values.get(j) || [0];
-      topSeries.values.toArray()[j] -= bottomPoint;
+      const bottomPoint = bottomSeries.values[j] || [0];
+      topSeries.values[j] -= bottomPoint;
     }
   }
 
   return seriesList;
 }
 
-function sortSeriesByLabel(s1: DataFrame, s2: DataFrame): number {
+export function sortSeriesByLabel(s1: DataFrame, s2: DataFrame): number {
   let le1, le2;
 
   try {
     // fail if not integer. might happen with bad queries
-    le1 = parseSampleValue(s1.name ?? '');
-    le2 = parseSampleValue(s2.name ?? '');
+    le1 = parseSampleValue(s1.name ?? s1.fields[1].name);
+    le2 = parseSampleValue(s2.name ?? s2.fields[1].name);
   } catch (err) {
     console.error(err);
     return 0;
